/****************************************************************************
 *
 * Project: --------------
 *
 * Copyright by Olimex Ltd. All rights reserved.
 *
 * File: rtctest.c
 * Description: RTC test
 * Developer: Dimitar Dimitrov ( dinuxbg,gmail.com )
 *
 * Last change: $Date: 2009-03-30 15:40:06 +0300 (Mon, 30 Mar 2009) $
 * Revision: $Revision: 29 $
 * Id: $Id: rtctest.c 669 2009-03-30 12:40:06Z dimitar $
 *
 ****************************************************************************/

#include <assert.h>
#include <stddef.h>
#include <stdint.h>
#include <stdio.h>

#include "rtctest.h"

#include "stm32f10x_lib.h"
#include "platform_config.h"
#include "lcd-320x240.h"

/**
 * @file
 * RTC test
 *
 * @todo TODO - document me!
 *
 */


void RTC_Configuration(void)
{
  /* Enable PWR and BKP clocks */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR | RCC_APB1Periph_BKP, ENABLE);

  /* Allow access to BKP Domain */
  PWR_BackupAccessCmd(ENABLE);

  /* Reset Backup Domain */
  BKP_DeInit();

  /* Enable LSE */
  RCC_LSEConfig(RCC_LSE_ON);
  /* Wait till LSE is ready */
  while (RCC_GetFlagStatus(RCC_FLAG_LSERDY) == RESET)
  {}

  /* Select LSE as RTC Clock Source */
  RCC_RTCCLKConfig(RCC_RTCCLKSource_LSE);

  /* Enable RTC Clock */
  RCC_RTCCLKCmd(ENABLE);

  /* Wait for RTC registers synchronization */
  RTC_WaitForSynchro();

  /* Wait until last write operation on RTC registers has finished */
  RTC_WaitForLastTask();

  /* Enable the RTC Second */
  RTC_ITConfig(RTC_IT_SEC, ENABLE);

  /* Wait until last write operation on RTC registers has finished */
  RTC_WaitForLastTask();

  /* Set RTC prescaler: set RTC period to 1sec */
  //RTC_SetPrescaler(32767); /* RTC period = RTCCLK/RTC_PR = (32.768 KHz)/(32767+1) */
  RTC_SetPrescaler(3);

  /* Wait until last write operation on RTC registers has finished */
  RTC_WaitForLastTask();
}


static void rtc_init(void)
{

  if (BKP_ReadBackupRegister(BKP_DR1) != 0xA5A5)
  {
    /* Backup data register value is not correct or not yet programmed (when
       the first time the program is executed) */

    printf("RTC not yet configured....\r\n");

    /* RTC Configuration */
    RTC_Configuration();

    printf("RTC configured....\r\n");

    BKP_WriteBackupRegister(BKP_DR1, 0xA5A5);
  }
  else
  {
    /* Check if the Power On Reset flag is set */
    if (RCC_GetFlagStatus(RCC_FLAG_PORRST) != RESET)
    {
      printf("\r\n\n Power On Reset occurred....\r\n");
    }
    /* Check if the Pin Reset flag is set */
    else if (RCC_GetFlagStatus(RCC_FLAG_PINRST) != RESET)
    {
      printf("\r\n\n External Reset occurred....\r\n");
    }

    //printf("\r\n No need to configure RTC....");
    /* Wait for RTC registers synchronization */
    RTC_WaitForSynchro();

    /* Enable the RTC Second */
    RTC_ITConfig(RTC_IT_SEC, ENABLE);
    /* Wait until last write operation on RTC registers has finished */
    RTC_WaitForLastTask();
  }
  /* Clear reset flags */
  RCC_ClearFlag();
}


void rtctest_sec_int(void)
{
}


int rtctest(void)
{  
	const unsigned int duration_ms = 5000;
	const unsigned int range_ms = 20;
	const unsigned int rtc_int_freq = 32768/4;
	u32 t0, t;
	unsigned int n;
	
	rtc_init();
	
	t0 = RTC_GetCounter();
	counter_1ms = 0;
	
	while(counter_1ms < duration_ms) {
		printf("\rRTC cntr=%d", RTC_GetCounter()/rtc_int_freq);
	}
	t = RTC_GetCounter();
	printf("\r                     \r");
	
	
	n = ((t-t0) * 1000) / rtc_int_freq;
	if((n<(duration_ms-range_ms)) || (n>(duration_ms+range_ms))) {
		printf("RTC OSCILLATOR\r\nERROR!\r\n\n");
		return -1;
	} else {
		printf("\r\n\n");
		return 0;
	}
}

